<?php

namespace NCSU\Auth\Framework\Laravel;

use Closure;
use Illuminate\Auth\AuthenticationException;
use Illuminate\Support\Facades\Auth;
use NCSU\Auth\Adapter\AdapterInterface;
use NCSU\Auth\Adapter\MockAuthAdapter;
use NCSU\Auth\Adapter\ShibAuthAdapter;
use NCSU\Auth\AuthService;
use NCSU\Auth\Http\Request;

class Authenticate
{
    /**
     * @var AuthService
     */
    private $authService;

    public function __construct(AdapterInterface $adapter = null, string $env = 'LARAVEL_USER')
    {
        if (!app()->environment('production')) {
            $adapter = new MockAuthAdapter(env($env));
        }

        if (!$adapter) {
            $adapter = new ShibAuthAdapter(Request::createFromGlobals());
        }

        $this->authService = new AuthService($adapter);
    }

    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request $request
     * @param  \Closure                 $next
     *
     * @return mixed
     * @throws AuthenticationException
     */
    public function handle($request, Closure $next)
    {
        if (Auth::guest()) {
            $response = $this->authService->authenticate();

            if (!$response->isValid()) {
                throw new AuthenticationException(implode(' ', $response->getMessages()));
            }

            $identity = $response->getIdentity();

            $user = $this->findOrCreateUser($identity);

            Auth::login($user);
        }

        return $next($request);
    }

    /**
     * Return user if exists; create and return if doesn't
     *
     * @param $shibbolethUser
     * @return User
     */
    private function findOrCreateUser($shibbolethUser)
    {
        $userModelClass = config('auth.providers.users.model');

        if ($authUser = $userModelClass::where('name', $shibbolethUser)->first()) {
            return $authUser;
        }

        return $userModelClass::create([
            'name' => $shibbolethUser,
            'email' => $shibbolethUser . '@ncsu.edu',
            'password' => bcrypt(str_random(64)),
        ]);
    }
}
